@@ -1,4 +1,5 @@
 import numpy as np
+import scipy as sp
 
 
 def _autocorr_func_1d(x, norm=True):
@@ -22,8 +23,10 @@ def _autocorr_func_1d(x, norm=True):
         n = n << 1
 
     # Compute the auto-correlation function using FFT
-    f = np.fft.fft(x - np.mean(x), n=2 * n)
-    acf = np.fft.ifft(f * np.conjugate(f))[: len(x)].real
+    #f = np.fft.fft(x - np.mean(x), n=2 * n)
+    #acf = np.fft.ifft(f * np.conjugate(f))[: len(x)].real
+    f = sp.fft.fft(x - np.mean(x), n=2 * n)
+    acf = sp.fft.ifft(f * np.conjugate(f))[: len(x)].real
     acf /= 4 * n
 
     # Normalize
@@ -33,21 +36,35 @@ def _autocorr_func_1d(x, norm=True):
     return acf
 
 
-def _autocorr_time_1d(y, c=5.0):
+def _autocorr_time_1d(y, c=5.0, method='mk'):
     """
     Integrated Autocorrelation Time (IAT) for 1-dimensional chain.
 
     Args:
-        y (array) : (nwalkers,nsteps) array for one parameter.
-        c (float) : Truncation parameter of automated windowing procedure of Sokal (1989), default is 5.0
+        y (array) : (nsteps, nwalkers) array for one parameter.
+        c (float) : Truncation parameter of automated windowing procedure of Sokal (1989), default is 5.0.
+        method (str) : Method to use to compute the IAT. Available options are ``mk`` (Default), ``dfm``, and ``gw``.
 
     Returns:
         The IAT of the chain y.
     """
-    f = np.zeros(y.shape[1])
-    for yy in y:
-        f += _autocorr_func_1d(yy)
-    f /= len(y)
+
+    if method not in ['mk', 'dfm', 'gw']:
+        raise ValueError('Please select one of the supported methods i.e. mk (Recommended), dfm, gw.')
+
+    if method == 'mk':
+        # Minas Karamanis method
+        f = _autocorr_func_1d(y.reshape((-1), order='C'))
+    elif method == 'dfm':
+        # Daniel Forman-Mackey method
+        f = np.zeros(y.shape[1])
+        for yy in y:
+            f += _autocorr_func_1d(yy)
+        f /= len(y)
+    else:
+        # Goodman-Weary method
+        f = _autocorr_func_1d(np.mean(y, axis=0))
+    
     taus = 2.0 * np.cumsum(f) - 1.0
 
     # Automated windowing procedure following Sokal (1989)
@@ -61,7 +78,7 @@ def _autocorr_time_1d(y, c=5.0):
     return taus[window]
 
 
-def AutoCorrTime(samples, c=5.0):
+def AutoCorrTime(samples, c=5.0, method='mk'):
     """
     Integrated Autocorrelation Time (IAT) for all the chains.
 
@@ -71,16 +88,22 @@ def AutoCorrTime(samples, c=5.0):
             3-dimensional array of shape (nsteps, nwalkers, ndim)
         c : float
             Truncation parameter of automated windowing procedure of Sokal (1989), default is 5.0
+        method : str
+            Method to use to compute the IAT. Available options are ``mk`` (Default), ``dfm``, and ``gw``.
 
     Returns
     -------
     taus : array
         Array with the IAT of all the chains.
     """
+
+    if method not in ['mk', 'dfm', 'gw']:
+        raise ValueError('Please select one of the supported methods i.e. mk (Recommended), dfm, gw.')
+
     _, _, ndim = np.shape(samples)
 
     taus = np.empty(ndim)
     for i in range(ndim):
-        taus[i] = _autocorr_time_1d(samples[:,:,i].T, c)
+        taus[i] = _autocorr_time_1d(samples[:,:,i].T, c, method)
 
     return taus
